﻿using Syncfusion.XForms.DataForm;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace formloop
{
    public class formpage : ContentPage
    {
        SfDataForm claimForm;
        static vm _vm;

        public formpage()
        {
            _vm = new vm();

            claimForm = new SfDataForm
            {
                HeightRequest = Device.RuntimePlatform == Device.iOS ? 250 : 150,
                LabelWidth = 2,
                EditorWidth = 2
            };
            claimForm.AutoGeneratingDataFormItem += GenerateFormItem;
            claimForm.ItemManager = new FormItemManager(claimForm);
            claimForm.RegisterEditor("ServiceType", "Picker");
            claimForm.RegisterEditor("MedicalExpenseType", "Picker");
            claimForm.RegisterEditor("Participant", "Picker");
            claimForm.RegisterEditor("CarePlanTask", "Picker");
            claimForm.RegisterEditor("NoDocumentation", "Switch");
            claimForm.CommitMode = CommitMode.LostFocus;
            claimForm.NotifyPropertyChanges = true;
        }
        protected override async void OnAppearing()
        {
            claimForm.DataObject = _vm.CdaClaim;
            var layout = new StackLayout
            {
                Margin = new Thickness(10, 20, 10, 20),
                Spacing = 5,
            };
            var pageLayout = new RelativeLayout() { IsClippedToBounds = true };

            var reimbursementLabel = new Label()
            {
                Margin = new Thickness(0, 0, 0, 10),
                Text = "Reimbursement Request",
                HorizontalOptions = LayoutOptions.Center
            };

            var termsGrid = new Grid()
            {
                Margin = new Thickness(10, 20, 10, 20),
                RowDefinitions =
                {
                    new RowDefinition {Height = new GridLength(1, GridUnitType.Auto) },
                    new RowDefinition {Height = new GridLength(1, GridUnitType.Auto) },
                },
                ColumnDefinitions =
                {
                    new ColumnDefinition {Width = new GridLength(.1, GridUnitType.Star) },
                    new ColumnDefinition {Width = new GridLength(.9, GridUnitType.Star) },
                },
            };
            var termsAgreed = new Label
            {
                Text = "\u2713",
                FontSize = 24,
                Opacity = .5,
                VerticalOptions = LayoutOptions.Start,
            };
            termsGrid.Children.Add(termsAgreed, 0, 0);
            termsGrid.Children.Add(new Label
            {
                Text = (string.IsNullOrEmpty(_vm.CdaClaim.ServiceType) || _vm.CdaClaim.ServiceType == "Dependent Care" || _vm.CdaClaim.ServiceType == "Commuter Transit" || _vm.CdaClaim.ServiceType == "Commuter Parking") ?
                                        string.Format("I am submitting a valid reimbursement request that has not been reimbursed through any other means. I understand I cannot deduct this expense on my tax return and that I must meet all IRS rules regarding reimbursement of allowable expenses. (Tap checkmark to attest)") :
                                        string.Format("I am submitting a valid reimbursement request for an eligible member covered by the {0} minimum essential coverage (MEC) health plan shown in Settings. This expense has not been reimbursed and will not be reimbursed by any health plan or other reimbursement arrangement. (Tap checkmark to attest)"),
                FontSize = 14,
                VerticalOptions = LayoutOptions.Start,
            }, 1, 0);
            var termsTap = new TapGestureRecognizer();
            termsTap.Tapped += async (s, ev) =>
            {
                termsAgreed.Opacity = .5;
                await Task.Delay(250);
                termsAgreed.Opacity = 1;
                _vm.TermsAgreedTo = true;
            };
            termsAgreed.GestureRecognizers.Add(termsTap);

            var documentsLabel = new Label()
            {
               
                Margin = new Thickness(0, 0, 0, 10),
                Text = "Substantiation",
                HorizontalOptions = LayoutOptions.Center
            };
            var docIntro = new Label()
            {
               
                Margin = new Thickness(15, 5, 10, 5),
                Text = _vm.CdaClaim.ServiceType != "Health Premium" ? "Snap or select photo(s) of documentation that supports your reimbursement request." : "Strongly recommended: Snap or select photo(s) of documentation that supports your reimbursement request.",
                HeightRequest = 50,
            };

            layout.Children.Add(reimbursementLabel);
            layout.Children.Add(claimForm);
            layout.Children.Add(termsGrid);
            layout.Children.Add(documentsLabel);
            layout.Children.Add(docIntro);

            pageLayout.Children.Add(new ScrollView() { Content = layout },
                Constraint.Constant(0),
                Constraint.Constant(0),
                Constraint.RelativeToParent((parent) => { return parent.Width; }),
                Constraint.RelativeToParent((parent) => { return parent.Height; }));       
            Content = pageLayout;
        }

        private void GenerateFormItem(object sender, AutoGeneratingDataFormItemEventArgs e)
        {
            (sender as SfDataForm).VerticalOptions = LayoutOptions.Fill;
            if (e.DataFormItem != null && e.DataFormItem.Name == "ServiceType")
            {
                var list = new List<string>
                {
                    "Health Premium",
                    "Medical Expense",
                };
                (e.DataFormItem as DataFormPickerItem).ItemsSource = list;
            }
            else if (e.DataFormItem != null && e.DataFormItem.Name == "Note")
            {
                e.DataFormItem.LabelPosition = LabelPosition.Top;
            }
            else if (e.DataFormItem != null && e.DataFormItem.Name == "NoDocumentation")
            {
                e.DataFormItem.LayoutOptions = LayoutType.TextInputLayout;
            }
            else if (e.DataFormItem != null && e.DataFormItem.Name == "MedicalExpenseType")
            {
                var list = new List<string>
                {
                    "Medical Service",
                    "Rx",
                    "Dental",
                    "Vision",
                    "Over the Counter Drug",
                    "Other"
                };
                (e.DataFormItem as DataFormPickerItem).ItemsSource = list;
            }
            else if (e.DataFormItem != null && e.DataFormItem.Name == "CostSharingType")
            {
                var list = new List<string>
                {
                    "Deductible",
                    "Copayment",
                    "Coinsurance"
                };
                (e.DataFormItem as DataFormPickerItem).ItemsSource = list;
            }
            if (e.DataFormItem != null)
            {
                switch (e.DataFormItem.Name)
                {
                    case "ServiceType":
                        e.DataFormItem.Order = 0;
                        if ((e.DataFormItem.EditorView as Picker)?.SelectedItem as string != "Medical Expense")
                        {
                            _vm.DisplayMedicalPicker = false;
                        }
                        break;
                    case "MedicalExpenseType":
                        e.DataFormItem.Order = 1;
                        if (!_vm.DisplayMedicalPicker)
                        {
                            e.Cancel = true;
                        }
                        break;
                }
            }
        }

        public class FormItemManager : DataFormItemManager
        {
            readonly SfDataForm sfDataForm;
            bool DisplayMedicalPickerOldvalue;
            
            public FormItemManager(SfDataForm dataForm) : base(dataForm)
            {
                sfDataForm = dataForm;
                
            }
            public override void SetValue(DataFormItem dataFormItem, object value)
            {
                if (dataFormItem.Name == "ServiceType")
                {
                    DisplayMedicalPickerOldvalue = _vm.DisplayMedicalPicker;
                    _vm.DisplayMedicalPicker = false;
                    _vm.DisplayNote = false;
                    if ((string)value == "Medical Expense")
                    {
                        _vm.DisplayNote = true;
                        _vm.DisplayMedicalPicker = true;
                    }
                    base.SetValue(dataFormItem, value);
                    if (DisplayMedicalPickerOldvalue != _vm.DisplayMedicalPicker)
                        sfDataForm.RefreshLayout(true);
                }
                else
                {
                    base.SetValue(dataFormItem, value);
                }
            }
            
            
        }


        protected override void OnDisappearing()
        {
        }
    }

    public class CdaLedgerEntryDto
    {
        [Display(AutoGenerateField = false)]
        public string AccountType { get; set; }
        [Display(AutoGenerateField = false)]
        public string UniqueId { get; set; }
        [Display(AutoGenerateField = false)]
        public string LedgerItemCode { get; set; }
        [Display(Name = "Expense Category")]
        public string ServiceType { get; set; }
        [Display(Name = "Medical Expense Type")]
        public string MedicalExpenseType { get; set; }
        [Display(AutoGenerateField = false)]
        public string CostSharingType { get; set; }
        [Display(Name = "Claim Date")]
        public DateTime? When { get; set; }
        [Display(AutoGenerateField = false)]
        public bool IsCredit { get; set; }
        [Display(AutoGenerateField = false)]
        public bool IsConsumerTx { get; set; }
        [Display(AutoGenerateField = false)]
        public string Description { get; set; }
        [Display(Name = "Describe Expense")]
        [DataType(DataType.MultilineText)]
        public string Note { get; set; }
        [Display(AutoGenerateField = false)]
        public string Comment { get; set; }
        [Display(Name = "Expense for")]
        public string Participant { get; set; }
        [Display(AutoGenerateField = false)]
        public string Relationship { get; set; }
        [Display(Name = "Careplan Task")]
        public string CarePlanTask { get; set; }
        [Display(AutoGenerateField = false)]
        public string Review { get; set; }
        [Display(AutoGenerateField = false)]
        public DateTime? ReviewDate { get; set; }
        [Display(AutoGenerateField = false)]
        public DateTime? ReviewDecisionDate { get; set; }
        [Display(Name = "Amount")]
        public decimal Amount { get; set; }
        [Display(AutoGenerateField = false)]
        public decimal ToPay { get; set; }
        [Display(AutoGenerateField = false)]
        public decimal Paid { get; set; }
        [Display(AutoGenerateField = false)]
        public decimal ToDeductible { get; set; }
        [Display(AutoGenerateField = false)]
        public decimal ToPrepayment { get; set; }
        [Display(AutoGenerateField = false)]
        public decimal Deferred { get; set; }
        [Display(AutoGenerateField = false)]

        public string Status { get; set; }
        [Display(AutoGenerateField = false)]
        public string StatusMessage { get; set; }
        [Display(AutoGenerateField = false)]
        public DateTime StatusDate { get; set; }
        [Display(AutoGenerateField = false)]
        public List<string> VerificationDocs { get; set; }
        [Display(Name = "Documentation not provided. Attest only.")]
        public bool NoDocumentation { get; set; }
        [Display(AutoGenerateField = false)]
        public string ReceiptDoc { get; set; }
    }
    public class vm
    {
        public vm()
        {
            CdaClaim = new CdaLedgerEntryDto();
        }
        CdaLedgerEntryDto _cdaClaim;
        public CdaLedgerEntryDto CdaClaim
        {
            get { return _cdaClaim; }
            set
            {
                _cdaClaim = value;
            }
        }

        public bool DisplayDependents { get; set; }
        public string SelectedService { get; set; }
        public bool DisplayNote { get; set; }
        public bool DisplayMedicalPicker { get; set; }

        public bool DisplayCostSharingPicker { get; set; }

        public bool ExpenseTypeSet { get; set; }
        public bool ParticipantSet { get; set; }
        public bool MedicalTypeSet { get; set; }
        public bool CostSharingTypeSet { get; set; }
        public bool ClaimDateSet { get; set; }

        private bool _displayNoDocs;
        public bool DisplayNoDocsSwitch
        {
            get
            {
                return _displayNoDocs;
            }
            set
            {
                _displayNoDocs = value;
            }
        }
        private bool _noDocs;
        public bool NoDocsAvailable
        {
            get
            {
                return _noDocs;
            }
            set
            {
                _noDocs = value;
            }
        }

        public bool TermsAgreedTo { get; set; }
    }
}
